@@ -201,9 +201,8 @@ fclustIndex <- function ( y, x, index= "all" )
     vecallindex[7] <- partition.entropy(clres)
   if (any(index==6) || (index==8))
     vecallindex[8] <- proportion.exponent(clres)
-  if (any(index==7) || (index==8)){
-    require(mva)
-    vecallindex[9] <- separation.index(clres,x)}
+  if (any(index==7) || (index==8))
+      vecallindex[9] <- separation.index(clres,x)
   
   names(vecallindex) <- c("fhv", "apd", "pd", "xb", "fs", "pc", "pe",
                           "pre", "si")
